@@ -6,7 +6,6 @@
 // where they get used as possible, so include files only get stuff that 
 // gets used in more than one file.
 //--------------------------------------------------------------------------
-
 typedef unsigned char uchar;
 
 #ifndef TRUE
@@ -28,6 +27,10 @@ typedef struct {
     unsigned Size;
 }Section_t;
 
+extern int ExifSectionIndex;
+
+extern int DumpExifMap;
+
 #define MAX_DATE_COPIES 10
 
 //--------------------------------------------------------------------------
@@ -62,10 +65,15 @@ typedef struct {
     int   LightSource;
     char  Comments[MAX_COMMENT];
 
-    unsigned char * ThumbnailPointer;  // Pointer at the thumbnail
-    unsigned ThumbnailSize;     // Size of thumbnail.
+    unsigned ThumbnailOffset;          // Exif offset to thumbnail
+    unsigned ThumbnailSize;            // Size of thumbnail.
+    unsigned LargestExifOffset;        // Last exif data referenced (to check if thumbnail is at end)
 
-    char * DateTimePointers[MAX_DATE_COPIES];
+    char  ThumbnailAtEnd;              // Exif header ends with the thumbnail
+                                       // (we can only modify the thumbnail if its at the end)
+    int   ThumbnailSizeOffset;
+
+    int  DateTimeOffsets[MAX_DATE_COPIES];
     int  numDateTimeTags;
 
     int GpsInfoPresent;
@@ -75,6 +83,7 @@ typedef struct {
 }ImageInfo_t;
 
 
+
 #define EXIT_FAILURE  1
 #define EXIT_SUCCESS  0
 
@@ -89,11 +98,12 @@ typedef enum {
 // prototypes for jhead.c functions
 void ErrFatal(char * msg);
 void ErrNonfatal(char * msg, int a1, int a2);
+void FileTimeAsString(char * TimeStr);
 
 // Prototypes for exif.c functions.
 int Exif2tm(struct tm * timeptr, char * ExifTime);
 void process_EXIF (unsigned char * CharBuf, unsigned int length);
-int RemoveThumbnail(unsigned char * ExifSection, unsigned int Length);
+int RemoveThumbnail(unsigned char * ExifSection);
 void ShowImageInfo(int ShowFileInfo);
 void ShowConciseImageInfo(void);
 const char * ClearOrientation(void);
@@ -102,6 +112,8 @@ double ConvertAnyFormat(void * ValuePtr, int Format);
 int Get16u(void * Short);
 unsigned Get32u(void * Long);
 int Get32s(void * Long);
+void Put32u(void * Value, unsigned PutValue);
+void create_EXIF(void);
 
 //--------------------------------------------------------------------------
 // Exif format descriptor stuff
@@ -139,7 +151,8 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode);
 void DiscardData(void);
 void DiscardAllButExif(void);
 int ReadJpegFile(const char * FileName, ReadMode_t ReadMode);
-int TrimExifFunc(void);
+int ReplaceThumbnail(const char * ThumbFileName);
+int SaveThumbnail(char * ThumbFileName);
 int RemoveSectionType(int SectionType);
 int RemoveUnknownSections(void);
 void WriteJpegFile(const char * FileName);
